#!/usr/bin/env python

from journalism.columns import TextColumn, IntColumn, FloatColumn
from journalism.exceptions import *
from journalism.table import Table

def save():
    raise NotImplementedError
